@@ -26,7 +26,6 @@
 #include "gsettingsbackendinternal.h"
 #include "gsimplepermission.h"
 #include "giomodule-priv.h"
-#include "gio-marshal.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -45,6 +44,13 @@ struct _GSettingsBackendPrivate
   GStaticMutex lock;
 };
 
+/* For g_settings_backend_sync_default(), we only want to actually do
+ * the sync if the backend already exists.  This avoids us creating an
+ * entire GSettingsBackend in order to call a do-nothing sync()
+ * operation on it.  This variable lets us avoid that.
+ */
+static gboolean g_settings_has_backend;
+
 /**
  * SECTION:gsettingsbackend
  * @title: GSettingsBackend
@@ -184,7 +190,7 @@ g_settings_backend_watch_weak_notify (gpointer  data,
  * that appears as an argument to some of the callbacks, you *must* have
  * @context as %NULL.  Otherwise, you are subject to cross-thread
  * dispatching and whatever owned @origin_tag at the time that the event
- * occured may no longer own it.  This is a problem if you consider that
+ * occurred may no longer own it.  This is a problem if you consider that
  * you may now be the new owner of that address and mistakenly think
  * that the event in question originated from yourself.
  *
@@ -372,7 +378,7 @@ g_settings_backend_dispatch_signal (GSettingsBackend *backend,
  * dispatching the signal later.
  *
  * The implementation may call this function at any other time it likes
- * in response to other events (such as changes occuring outside of the
+ * in response to other events (such as changes occurring outside of the
  * program).  These calls may originate from a mainloop or may originate
  * in response to any other action (including from calls to
  * g_settings_backend_write()).
@@ -401,7 +407,7 @@ g_settings_backend_changed (GSettingsBackend *backend,
  * g_settings_backend_keys_changed:
  * @backend: a #GSettingsBackend implementation
  * @path: the path containing the changes
- * @items: the %NULL-terminated list of changed keys
+ * @items: (array zero-terminated=1): the %NULL-terminated list of changed keys
  * @origin_tag: the origin tag
  *
  * Signals that a list of keys have possibly changed.  Backend
@@ -609,9 +615,11 @@ g_settings_backend_flatten_one (gpointer key,
 /**
  * g_settings_backend_flatten_tree:
  * @tree: a #GTree containing the changes
- * @path: the location to save the path
- * @keys: the location to save the relative keys
- * @values: the location to save the values, or %NULL
+ * @path: (out): the location to save the path
+ * @keys: (out) (transfer container) (array zero-terminated=1): the
+ *        location to save the relative keys
+ * @values: (out) (allow-none) (transfer container) (array zero-terminated=1):
+ *          the location to save the values, or %NULL
  *
  * Calculate the longest common prefix of all keys in a tree and write
  * out an array of the key names relative to that prefix and,
@@ -909,6 +917,13 @@ g_settings_backend_class_init (GSettingsBackendClass *class)
   g_type_class_add_private (class, sizeof (GSettingsBackendPrivate));
 }
 
+static void
+g_settings_backend_variant_unref0 (gpointer data)
+{
+  if (data != NULL)
+    g_variant_unref (data);
+}
+
 /*< private >
  * g_settings_backend_create_tree:
  * @returns: a new #GTree
@@ -921,12 +936,12 @@ GTree *
 g_settings_backend_create_tree (void)
 {
   return g_tree_new_full ((GCompareDataFunc) strcmp, NULL,
-                          g_free, (GDestroyNotify) g_variant_unref);
+                          g_free, g_settings_backend_variant_unref0);
 }
 
 /**
  * g_settings_backend_get_default:
- * @returns: the default #GSettingsBackend
+ * @returns: (transfer full): the default #GSettingsBackend
  *
  * Returns the default #GSettingsBackend. It is possible to override
  * the default by setting the <envar>GSETTINGS_BACKEND</envar>
@@ -980,6 +995,7 @@ g_settings_backend_get_default (void)
 
       extension_type = g_io_extension_get_type (extension);
       instance = g_object_new (extension_type, NULL);
+      g_settings_has_backend = TRUE;
 
       g_once_init_leave (&backend, (gsize) instance);
     }
@@ -1019,12 +1035,15 @@ g_settings_backend_get_permission (GSettingsBackend *backend,
 void
 g_settings_backend_sync_default (void)
 {
-  GSettingsBackendClass *class;
-  GSettingsBackend *backend;
+  if (g_settings_has_backend)
+    {
+      GSettingsBackendClass *class;
+      GSettingsBackend *backend;
 
-  backend = g_settings_backend_get_default ();
-  class = G_SETTINGS_BACKEND_GET_CLASS (backend);
+      backend = g_settings_backend_get_default ();
+      class = G_SETTINGS_BACKEND_GET_CLASS (backend);
 
-  if (class->sync)
-    class->sync (backend);
+      if (class->sync)
+        class->sync (backend);
+    }
 }
